#!/bin/bash

SRC=${SRC:-imagestore:/srv/cloud/objectstore/images}
DEST=${DEST:-/srv/cloud/objectstore/images}

# call with VERBOSE=v in the environment to see verbose messages
VERBOSE=${VERBOSE:-}

LOCKFILE=${DEST}/imgcopy-lock.pid

function lock() {
    set -o noclobber
    echo $$ > ${LOCKFILE}
    return $?
}

function unlock() {
    rm ${LOCKFILE}
    exit
}

if ! lock ; then
    oldpid=`cat ${LOCKFILE}`
    echo "Unable to lock: is ${oldpid} still running?"
    exit
fi
trap unlock INT TERM

rsync -urtz${VERBOSE} --include '*/' --include '*/image' --exclude='*' ${SRC} ${DEST}

for x in `find ${DEST} -name image | sed -e 's~'${DEST}'~~' -e 's/image$/info.json/'` ; do
    rsync -utz${verbose} ${SRC}/${x} ${DEST}/${x}
done

unlock
